@@ -12,12 +12,14 @@ let failwith s = Trace.make_failwith name s
 (**************************************************************)
 
 type typ =
-    (* These events should have messages associated with them. *)
-  | ECast				(* Multicast message *)
-  | ESend				(* Pt2pt message *)
-  | ESubCast				(* Multi-destination message *)
-  | ECastUnrel				(* Unreliable multicast message *)
-  | ESendUnrel				(* Unreliable pt2pt message *)
+    (* These events have messages associated with them. *)
+  | ECast of Iovecl.t			(* Multicast message *)
+  | ESend of Iovecl.t			(* Pt2pt message *)
+  | ECastUnrel of Iovecl.t   	        (* Unreliable multicast message *)
+  | ESendUnrel of Iovecl.t		(* Unreliable pt2pt message *)
+
+  (* These events have Buf.t messages, not iovec's. 
+   *)
   | EMergeRequest			(* Request a merge *)
   | EMergeGranted			(* Grant a merge request *)
   | EMergeDenied			(* Deny a merge request *)
@@ -67,11 +69,23 @@ type typ =
 
 (**************************************************************)
 
+type compact_typ = 
+  | C_ECast
+  | C_ESend
+  | C_EOther
+
+let compact_typ_of_typ = function
+  | ECast _ -> C_ECast 
+  | ESend _ -> C_ESend
+  | _ -> C_EOther
+      
+
+(**************************************************************)
+
 type field =
       (* Common fields *)
   | Type	of typ			(* type of the message*)
   | Peer	of rank			(* rank of sender/destination *)
-  | Iov	        of Iovecl.t		(* payload of message *)
   | ApplMsg				(* was this message generated by an appl? *)
 
       (* Uncommon fields *)
@@ -130,8 +144,8 @@ type field =
 
 type t = {
   typ	 : typ ;
+  c_type : compact_typ ;
   peer   : rank ;
-  iov	 : Iovecl.t ;
   applmsg : bool ;
   extend : field list
 }
@@ -151,85 +165,95 @@ let invalid_rank = -1
 
 (**************************************************************)
 
-let bodyCore debug typ peer iov = {
+let bodyCore debug typ peer = {
   typ 	  = typ ;
+  c_type  = compact_typ_of_typ typ;
   peer	  = peer ;
   applmsg = false ;
-  iov	  = iov ;
   extend  = []
 }
 
 (**************************************************************)
 
-let castIov debug iov = {
-  typ 	  = ECast ;
-  peer    = invalid_rank ;
-  applmsg = false ;
-  iov	  = iov ;
-  extend  = []
-}
+let castIov debug iov = 
+  let typ = ECast iov in {
+    typ     = typ ;
+    c_type  = compact_typ_of_typ typ ;
+    peer    = invalid_rank ;
+    applmsg = false ;
+    extend  = []
+  }
 
 (**************************************************************)
 
-let castIovAppl debug iov = {
-  typ 	  = ECast ;
-  peer    = invalid_rank ;
-  applmsg = true ;
-  iov	  = iov ;
-  extend  = []
-}
+let castIovAppl debug iov = 
+  let typ = ECast iov in {
+    typ     = typ ;
+    c_type  = compact_typ_of_typ typ ;
+    peer    = invalid_rank ;
+    applmsg = true ;
+    extend  = []
+  }
+			   
+(**************************************************************)
+
+let sendPeerIovAppl debug rank iov =
+  let typ = ESend iov in {
+    typ     = typ ;
+    c_type  = compact_typ_of_typ typ ;
+    peer    = rank ;
+    applmsg = true ;
+    extend  = []
+  }
 
 (**************************************************************)
 
-let sendPeerIovAppl debug rank iov = {
-  typ 	  = ESend ;
-  peer 	  = rank ;
-  applmsg = true ;
-  iov	  = iov ;
-  extend  = []
-}
-
+let sendPeerIov debug rank iov = 
+  let typ = ESend iov in {
+    typ     = typ ;
+    c_type  = compact_typ_of_typ typ ;
+    peer    = rank ;
+    applmsg = false ;
+    extend  = []
+  }
+			   
 (**************************************************************)
-
-let sendPeerIov debug rank iov = {
-  typ 	  = ESend ;
-  peer 	  = rank ;
-  applmsg = false ;
-  iov	  = iov ;
-  extend  = []
-}
-
-(**************************************************************)
-
-let rec setLoop iov pee app typ ext usl = match usl with
-  | [] -> {peer=pee;typ=typ;iov=iov;applmsg=app;extend=ext}
+			   
+let rec setLoop pee app typ ext usl = match usl with
+  | [] -> {peer=pee;typ=typ;c_type=compact_typ_of_typ typ;applmsg=app;extend=ext}
   | hd :: tl -> match hd with
-    | Type   typ -> setLoop iov pee app typ ext tl
-    | Peer   pee -> setLoop iov pee app typ ext tl
-    | Iov    iov -> setLoop iov pee app typ ext tl
-    | ApplMsg    -> setLoop iov pee true typ ext tl
-    | f          -> setLoop iov pee app typ (f::ext) tl
+    | Type   typ -> setLoop pee app typ ext tl
+    | Peer   pee -> setLoop pee app typ ext tl
+    | ApplMsg    -> setLoop pee true typ ext tl
+    | f          -> setLoop pee app typ (f::ext) tl
 
-let set debug {typ=typ;iov=iov;peer=pee;applmsg=app;extend=ext} dsl =
-  setLoop iov pee app typ ext dsl
+let set debug {typ=typ;peer=pee;applmsg=app;extend=ext} dsl =
+  setLoop pee app typ ext dsl
+
+let set_typ debug ev typ = {ev with typ=typ; c_type=compact_typ_of_typ typ}
+
+let setSendIovFragment debug ev iov = 
+  let typ = ESend iov in
+  let c_type = compact_typ_of_typ typ in
+  {ev with typ=typ;c_type=c_type;extend=Fragment::ev.extend}
+
+let setCastIovFragment debug ev iov = 
+  let typ = ECast iov in
+  let c_type = compact_typ_of_typ typ in
+  {ev with typ=typ;c_type=c_type;extend=Fragment::ev.extend}
 
-let setIovFragment debug ev iov = {ev with iov=iov;extend=Fragment::ev.extend}
 let setNoTotal debug ev = {ev with extend=NoTotal::ev.extend}
 let setPeer debug ev peer = {ev with peer=peer}
-let setSendUnrelPeer debug ev peer = {ev with typ=ESendUnrel;peer=peer}
+
+let setSendUnrelPeer debug ev peer = 
+  let typ = ESendUnrel Iovecl.empty in
+  let c_type = compact_typ_of_typ typ in
+  {ev with typ=typ; c_type=c_type; peer=peer}
 
 (**************************************************************)
 
 let create debug typ fields =
-  setLoop Iovecl.empty invalid_rank false typ [] fields
-
-(*
-let free debug ev = Iovecl.free (Refcnt.info name debug) ev.iov
-let copy debug ev = Iovecl.ref (Refcnt.info name debug) ev.iov ; ev
-*)
-let free debug ev = Iovecl.free ev.iov
-let free_noIov = ignore2
-let copy debug ev = { ev with iov = Iovecl.copy ev.iov }
+  setLoop invalid_rank false typ [] fields
 
 let upCheck debug ev = () (*Iovecl.check ev.iov*)
 let dnCheck debug ev = () (*Iovecl.check ev.iov*)
@@ -265,9 +289,14 @@ let getExtendOpt e f =
   in loop e.extend
 
 let getExtend  e = e.extend
-let getIov     e = e.iov
+
+let getIov     e = match e.typ with 
+  | ECast iov | ESend iov | ECastUnrel iov | ESendUnrel iov -> Some iov
+  | _ -> None
+
 let getPeer    e = e.peer
 let getType    e = e.typ
+let getCompactType e = e.c_type
 let getApplMsg e = e.applmsg
 
 let getDbgName       = getExtendFail (function DbgName i       -> Some i | _ -> None) "DbgName"
@@ -304,9 +333,17 @@ let getAgreedKey = getExtendFail (function (AgreedKey a) -> Some a | _ -> None)
 let getRekeyFlag = getExtendFail  (function RekeyFlag flg -> Some flg | _ -> None) "RekeyFlag"
 (**************************************************************)
 
-let getIovLen ev =
-  let iov = getIov ev in
-  Iovecl.len iov
+let free debug ev = 
+  match getIov ev with 
+    | None -> ()
+    | Some iovl -> Iovecl.free iovl
+
+let copy debug ev = 
+  match getIov ev with
+    | None -> ev
+    | Some iovl -> 
+	ignore (Iovecl.copy iovl) ;
+	ev 
 
 (**************************************************************)
 
@@ -315,8 +352,8 @@ let string_of_type = Trace.debug "" (function
   | EAsync	  	-> "EAsync"
   | EBlock	  	-> "EBlock"
   | EBlockOk	  	-> "EBlockOk"
-  | ECast	  	-> "ECast"
-  | ECastUnrel	  	-> "ECastUnrel"
+  | ECast	_  	-> "ECast"
+  | ECastUnrel	_  	-> "ECastUnrel"
   | EDump	  	-> "EDump"
   | EElect		-> "EElect"
   | EExit	  	-> "EExit"
@@ -338,11 +375,10 @@ let string_of_type = Trace.debug "" (function
   | ERekey              -> "ERekey"
   | ERekeyPrcl          -> "ERekeyPrcl"
   | ERekeyPrcl2         -> "ERekeyPrcl2"
-  | ESend	  	-> "ESend"
-  | ESendUnrel	  	-> "ESendUnrel"
+  | ESend _	  	-> "ESend"
+  | ESendUnrel	_  	-> "ESendUnrel"
   | EStable	  	-> "EStable"
   | EStableReq	  	-> "EStableReq"
-  | ESubCast            -> "ESubCast"
   | ESuspect	  	-> "ESuspect"
   | ESystemError 	-> "ESystemError"
   | ETimer	  	-> "ETimer"
@@ -421,11 +457,13 @@ let to_string = Trace.debug "" (fun ev ->
 
 (**************************************************************)
 
-let castEv	  debug	     = create debug ECast[]
-let castUnrel	  debug	     = create debug ECastUnrel[]
-let castPeerIov   debug o i  = create debug ECast[Peer o;Iov i]
-let sendPeer      debug d    = create debug ESend[Peer d]
-let sendUnrelPeer debug d    = create debug ESendUnrel[Peer d]
+let castEv        debug =        create debug (ECast Iovecl.empty) []
+let castUnrelIov  debug	iov    = create debug (ECastUnrel iov) []
+let castUnrel	  debug	       = create debug (ECastUnrel Iovecl.empty) []
+let sendPeer      debug d      = create debug (ESend Iovecl.empty) [Peer d]
+let castPeerIov   debug o iov  = create debug (ECast iov) [Peer o]
+let sendUnrelPeer debug d      = create debug (ESendUnrel Iovecl.empty) [Peer d]
+let sendUnrelPeerIov debug d iov  = create debug (ESendUnrel iov) [Peer d]
 let suspectReason debug s r  = create debug ESuspect[Suspects s;SuspectReason r]
 let timerAlarm    debug t    = create debug ETimer[Alarm t]
 let timerTime     debug t    = create debug ETimer[Time t]
